<?php
$action['content type'] = 'html';
$action['status'] = 200;



// Prefix URL with correct paths
// FLAG duplication from web/template
function manipulate_urls ($string, $path = '') {

	global $paths;
	global $app;

	// Internal URLs (from app package root)
	$string = preg_replace('/(src|href)\s*=\s*[\"\'](?:\/)([^"\']*)[\"\']/U', '\\1'.'="'.$paths['domain']['apps'].$app['id'].'/'.'\\2"', $string);

	// Use endpoint (special case, reconvert the relative URLs I just captured)
	// FLAG hardcoded 'use/', need to parse it from $paths
	$string = str_replace('href="'.$paths['domain']['apps'].$app['id'].'/use/', 'href="'.$paths['domain']['use'].$app['id'].'/', $string);
	$string = str_replace('src="'.$paths['domain']['apps'].$app['id'].'/use/', 'src="'.$paths['domain']['use'].$app['id'].'/', $string);

	// Internal relative URLs (within the current path)
	$string = preg_replace('/(src|href)\s*=\s*[\"\'](?!\/|http|ftp|#|mailto:)([^"\']*)[\"\']/U', '\\1'.'="'.$paths['domain']['apps'].$app['id'].'/'.$path.'\\2"', $string);

	return $string;
}



// Package contents
$app = array_merge($app, package_map_contents($app));

// Output
$action['output'] = '
<!DOCTYPE html>
<html lang="'.(isset($app['languages']) ? $app['languages'][0] : $settings['language']).'">

	<head>

		<meta charset="'.$settings['character set'].'">
		';



		// Viewport scaling
		if (isset($app['viewport']) and !empty($app['viewport'])) {
			$temp = array('css' => '', 'meta' => '');

			// CSS definitions
			foreach ($app['viewport'] as $key => $value) {
				$temp['css'] .= $key.':'.$value.';';
			}

			// Meta tag definitions

			// Convert dimension definitions from CSS
			foreach (array(
				'width' => array('width', 'min-width', 'max-width'),
				'height' => array('height', 'min-height', 'max-height')
			) as $key => $value) {
				foreach ($value as $key2 => $value2) {
					if (isset($app['viewport'][$value2])) {
						$temp['meta'] .= $key.'='.$app['viewport'][$value2].',';
						break;
					}
				}
			}

			// Convert scaling-related definitions from CSS
			foreach (array(
				'zoom' => 'initial-scale',
				'min-zoom' => 'minimum-scale',
				'max-zoom' => 'maximum-scale'
			) as $key => $value) {

				// Only integer values are included
				// FLAG convert percentages into integer format
				if (isset($app['viewport'][$key]) and (is_int($app['viewport'][$key]) or is_float($app['viewport'][$key]))) {
					$temp['meta'] .= $value.'='.$app['viewport'][$key].',';
				}
			}

			// User-scaling
			if (isset($app['viewport']['user-zoom']) and $app['viewport']['user-zoom'] == 'fixed') {
				$temp['meta'] .= 'user-scalable=no,';
			}

			// Output scaling definitions
			$action['output'] .= '
			<meta name="viewport" content="'.dont_end_with($temp['meta'], ',').'">
			<style type="text/css">
				@-ms-viewport{'.$temp['css'].'}
				@-o-viewport{'.$temp['css'].'}
				@viewport{'.$temp['css'].'}
			</style>
			';
		}



		// Meta info from app
		if (isset($app['meta']) and !empty($app['meta'])) {

			// Title
			if (isset($app['meta']['name'])) {
				$action['output'] .= '
				<title>'.$app['meta']['name'].'</title>
				<meta name="application-name" content="'.$app['meta']['name'].'">
				';
			}
			
			// Description
			if (isset($app['meta']['decription'])) {
				$action['output'] .= '<meta name="decription" content="'.$app['meta']['decription'].'">';
			}

			// Author
			if (isset($app['meta']['author'])) {
				$action['output'] .= '<meta name="author" content="'.$app['meta']['author'].'">';
			}

			// Icon from app package
			if (isset($app['meta']['icon'])) {
				if (file_exists($p['apps'].$app['id'].'/'.$app['meta']['icon'])) {
					$temp = detect($app['meta']['icon'], 'extension');
					$temp2 = $paths['domain']['apps'].$app['id'].'/'.$app['meta']['icon'];

					// Format support
					if (in_array($temp, array('png', 'gif', 'jpg'))) {
						$action['output'] .= '<link rel="icon" href="'.$temp2.'" type="'.$constants['content types'][$temp].'">';
					} else if ($temp == 'ico') {
						$action['output'] .= '<link rel="shortcut icon" href="'.$temp2.'" type="image/x-icon">';
					}
				}
			}

		}



		// External tool resources
		if (!empty($app['tools']['external'])) {
			foreach ($app['tools']['external'] as $value) {
				$temp = detect($value, 'extension');

				// Styles
				if (in_array($temp, $constants['app package']['file types']['style'])) {
					$action['output'] .= '<link rel="stylesheet" href="'.$value.'" media="all">';

				// Behavior
				} else if (in_array($temp, $constants['app package']['file types']['behavior'])) {
					$action['output'] .= '<script src="'.$value.'"></script>';
				}
			}
		}

		// Tools and app package contents come from other web actions
		$temp = $paths['domain']['use'].$app['id'].'/'.$action['parent'].'/';
		$action['output'] .= '
		<link rel="stylesheet" href="'.$temp.'style/" media="all">
		<script src="'.$temp.'behavior/"></script>
		';

		// Normalize document styling
		$action['output'] .= '
		<style type="text/css">
			html {
				min-height: 100%;
				margin: 0;
				padding: 0;
				border: 0;
			}
		</style>
		';

	// Body structure
	$action['output'] .= '
	</head>

	<!--
	'.(isset($app['meta']['name']) ? $app['meta']['name'] : 'This app').' is running on Proot. Check out http://eiskis.net/proot/ to try it out yourself.
	-->

	<body>
	';

	// Bootstrap app's root structure
	$temp = glob_files($p['apps'].$app['id'].'/', $constants['app package']['file types']['structure']);
	if (count($temp) !== 0) {
		foreach ($temp as $value) {
			$action['output'] .= '
			'.manipulate_urls(file_get_contents($value), '').'
			';
		}
	}

	// Executing scripts
	$action['output'] .= '
	<script type="text/javascript">

		// Fill in the URL components from the client request
		'.(!empty($input) ? 'app.url = ["'.implode('", "', $input).'"];' : '').'

		// Run normalization & bootstrapping scripts as fast as possible
		if (typeof p !== "undefined" && p.normalize) {
			p.normalize.all();
		}

		// Launch a full Proot app (i.e. when "p.app" is available)
		if (typeof jQuery !== "undefined" && typeof bl !== "undefined" && typeof p !== "undefined" && p.app) {
			p.app.launch("body");

		// No Proot JS library
		} else {
			app.launch(app, app.shared);
		}

	</script>
	';

	// Close
	$action['output'] .= '
	</body>

</html>
';
?>