<?php
// Gives out action's parameters based on given input & validated based on action configuration
$action = array(
	'parent' => $choices['action']['parent'],
	'id' => $choices['action']['id']
);



// App must support the action
// FLAG app manifest should be allowed to be more specific than this
if (!in_array($action['parent'], $app['actions'])) {
	$errors[] = '412 '.$app['id'].' does not support '.$action['parent'];
}



// All good
if (empty($errors)) {

	// Input handling and other preparations from action, if needed
	$filepath = $p['actions'].$action['parent'].'/'.$action['id'].'/'.$constants['action package']['configuration'].'.php';
	if (file_exists($filepath)) {
		include_once $filepath;



		// Dependencies
		if (isset($action['dependencies']) and !empty($action['dependencies'])) {
			foreach (to_array($action['dependencies']) as $key => $value) {

				// Sibling action missing
				if ((is_int($key) and is_string($value)) and !in_array($value, $available['actions'][$action['parent']])) {
					$errors[] = '412 this action requires '.$value.' for '.$action['parent'];

				// Actions from another group
				} else if (is_string($key)) {
					$value = to_array($value);

					// Group missing
					if (!isset($available['actions'][$key])) {
						$errors[] = '412 this action requires '.limplode(', ', $value, ' and ').' action'.(count($value) > 1 ? 's' : '').' for '.$key;

					// Go through required actions
					} else {
						foreach ($value as $id) {
							if (!in_array($id, $available['actions'][$key])) {
								$errors[] = '412 this action requires '.$id.' action for '.$key;
							}
						}

					}

				}

			}
		}



		// Required backend utilities
		if (empty($errors)) {

			if (isset($action['utilities']) and !empty($action['utilities'])) {
				foreach (to_array($action['utilities']) as $id) {

					// Individual file gets precedence
					$string = $p['utilities'].$id;
					if (is_file($string.'.php')) {
						include_once $string.'.php';

					// Complete directory
					} else if (is_dir($string)) {
						foreach (rglob_files($string.'/', array('php')) as $filepath) {
							include_once $filepath;
						}

					// Missing required utilities
					} else {
						$errors[] = '500 '.$action['id'].' '.$action['parent'].' requires the '.$id.' utility';
					}

				}
			}

		}



		// Validate possible input, reformat it for the action
		if (empty($errors)) {

			if (isset($action['input'])) {
				$temp = run_validations($input, $action['input']);
				
				// Handle errors in input
				if (isset($temp['errors'])) {
					$errors = array_merge($errors, $temp['errors']);
				} else {
					$input = $temp;
				}

			}
		}

	}



	// Run action
	if (empty($errors)) {

		// Should we check for cache'd response?
		$temp = true;
		if ($settings['cache'] > 0 and isset($action['cache']) and $action['cache'] > 0) {

			// Is there a valid response in cache?
			$directory = $p['cache'].$app['id'].'/'.$action['parent'].'/'.$action['id'].'/';
			$filepath = '';
			foreach (glob($directory.'cache.*'.dont_end_with('.'.array_serialize($input), '.')) as $value) {
				$filepath = $value;
				break;
			}

			// Non-expired cache file found
			if (!empty($filepath) and time() < filemtime($filepath) + (60 * min($settings['cache'], $action['cache']))) {

				// Change action content to output cache file
				$action['file output'] = (isset($action['cache file output']) and $action['cache file output'] == 'redirect') ? 'redirect' : 'read file';
				$action['output'] = $filepath;

				// Read original content type from filename
				$array = explode('.', substr(basename($filepath), 6));
				$action['content type'] = base64_decode($array[0]);

				$temp = false;
			}

		}



		// Let the action create a new response
		if ($temp) {

			// Custom validations
			$filepath = $p['actions'].$action['parent'].'/'.$action['id'].'/'.$constants['action package']['validation'].'.php';
			if (is_file($filepath)) {
				include_once $filepath;
			}

			// Let's stop if there were errors from custom validations
			if (empty($errors)) {

				// Business logic & output generation
				foreach ($constants['action package']['body'] as $id) {
					$filepath = $p['actions'].$action['parent'].'/'.$action['id'].'/'.$id.'.php';
					if (empty($errors) and file_exists($filepath)) {
						include_once $filepath;
					}
				}

				// Action needs to have created output at this point (empty output is fine, but it needs to be there)
				if (empty($errors) and !isset($action['output'])) {
					$errors[] = '500 no output from '.$action['id'];
				}

			}

		}

	}



}
?>