<?php
// Non-hardcoded values
$constants = array();

// The loose definition of app package format
$constants['app package'] = array(

	// Main files
	'manifest' 		=> 'app.json',					// App manifest
	'launch' 		=> 'app.js',					// App's launch method

	// Accepted directories for package contents
	'views'			=> 'views/',					// Views and similar packaged contents *
	'shared'		=> 'shared/',					// Shared code for all views
	'tools'			=> 'tools/',					// Tools included in app package

	// Supported content in app manifest
	'manifest content' => array(
		'meta',
		'viewport',
		'languages',
		'actions',
		'tools',
		'settings'
	),

	// Supported file types per category
	'file types' => array(
		'configuration' => array('json'),
		'behavior' 		=> array('js'),
		'structure' 	=> array('html'),
		'style' 		=> array('css', 'less')
	)
);

// Explicitly supported MIME type shorthands
$constants['content types'] = array(
	'txt' 			=> 'text/plain',

	'html' 			=> 'text/html',
	'xhtml' 		=> 'application/xhtml+xml',
	'json' 			=> 'application/json',
	'csv' 			=> 'text/csv',

	'js' 			=> 'application/javascript',
	'css' 			=> 'text/css',
	'xml' 			=> 'text/xml',
	'rss' 			=> 'application/rss+xml',

	'pdf' 			=> 'application/pdf',
	'zip' 			=> 'application/zip',

	'gif' 			=> 'image/gif',
	'jpg' 			=> 'image/jpeg',
	'png' 			=> 'image/png',
	'svg' 			=> 'image/svg+xml',
	'tif' 			=> 'image/tiff',

	'mpg' 			=> 'video/mpeg',

	'eot' 			=> 'application/vnd.ms-fontobject',
	'otf' 			=> 'font/otf',
	'ttf' 			=> 'font/ttf',
	'woff' 			=> 'font/x-woff'
);

// Supported HTTP status codes
$constants['status codes'] = array(
	100 => '100 Continue',
	200 => '200 OK',
	201 => '201 Created',
	204 => '204 No Content',
	302 => '302 Found',
	304 => '304 Not Modified',
	400 => '400 Bad Request',
	404 => '404 Not Found',
	406 => '406 Not Acceptable',
	410 => '410 Gone',
	412 => '412 Precondition Failed',
	415 => '415 Unsupported Media Type',
	418 => '418 I\'m a teapot',
	500 => '500 Internal Server Error',
	503 => '503 Service Unavailable'
);

// File type groups
$constants['file types'] = array(
	'image' 		=> array('tif', 'psd', 'eps', 'ai'),
	'webimage' 		=> array('jpg', 'png', 'gif', 'svg'),
	'bitmap' 		=> array('jpg', 'png', 'gif'),
	'audio' 		=> array('mp3', 'ogg', 'wma', 'wav', 'flac'),
	'video' 		=> array('flv', 'wmv', 'avi', 'mpg', 'mp4', 'mov', 'rm'),
	'pdf' 			=> array('pdf'),
	'document' 		=> array('doc', 'docx', 'dot', 'ppt', 'pot', 'odt', 'xls', 'rtf'),
	'webdocument' 	=> array('html', 'css', 'php', 'phps', 'txt', 'xml', 'rss')
);

// Supported validation formats as regular expressions or lists of accepted values
$constants['patterns'] = array(
	'base64' 		=> '^(?:[A-Za-z0-9+/]{4})*(?:[A-Za-z0-9+/]{2}==|[A-Za-z0-9+/]{3}=)?$',
	'content type' 	=> '#^[-\w]+/[-\w]+$#',
	'safename' 		=> '/\A[a-z0-9]*\z/i',
	'filename' 		=> '/\A[a-z0-9\-\_]*\z/i',
	'url' 			=> '|^http(s)?://[a-z0-9-]+(.[a-z0-9-]+)*(:[0-9]+)?(/.*)?$|i',
	'email' 		=> '/^[^0-9][a-zA-Z0-9_]+([.][a-zA-Z0-9_]+)*[@][a-zA-Z0-9_]+([.][a-zA-Z0-9_]+)*[.][a-zA-Z]{2,4}$/',
	'ip' 			=> '^([01]?\\d\\d?|2[0-4]\\d|25[0-5])\\.([01]?\\d\\d?|2[0-4]\\d|25[0-5])\\.([01]?\\d\\d?|2[0-4]\\d|25[0-5])\\.([01]?\\d\\d?|2[0-4]\\d|25[0-5])$'
);

// Content types for error responses per HTTP method
$constants['default content types'] = array(
	'get' 	=> 'html',
	'post' 	=> 'json',
	'put' 	=> 'json'
);

// Names of the files looked for in action packages
$constants['action package'] = array(
	'configuration' => 'configuration',
	'validation' 	=> 'validation',
	'body' 			=> array(
		'action',
		'output'
	)
);

?>