<?php
// Root of all Proot
error_reporting(E_ALL|E_STRICT);
ini_set('display_errors', '0');
ini_set('log_errors', '1');

// Get my bearings
require 'paths.php';

// Core helpers for server-side code
foreach (glob($p['helpers'].'*.php') as $filepath) {
	require $filepath;
}

// We run the server program in phases
$ids = array(
	'constants',		// 1. Global constants
	'settings',			// 2. System settings
	'available',		// 3. Lists of available items

	'request',			// 4. Take in original request headers & content
	'choices',			// 5. Select basic system parameters
	'input',			// 6. Validate and prioritize user input

	'app',				// 7. Extract the selected app from app package
	'action',			// 8. Run the selected backend action

	'response',			// 9. Decide on and prepare response
	'headers',			// 10. Compose response headers
	'output'			// 11. Composing response body content
);



// Execution

// Run each phase, stop after errors
$errors = array();
foreach ($ids as $id) {

	// Include core file as well as hacks
	foreach (array(
		'before' => $p['hacks'].'before_'.$id.'.php',
		'phase' => $p['core'].$id.'.php',
		'after' => $p['hacks'].'after_'.$id.'.php'
	) as $key => $filepath) {

		// Stop if there are errors
		if (empty($errors)) {

			// The actual phase is always required
			if ($key == 'phase') {

				// The actual phase must exist
				if (file_exists($filepath)) {
					require $filepath;

				// Forward to error output and stop if necessary
				} else {
					$errors[] = '500 '.$id.' phase is missing';
					require $p['core'].'error.php';
					break 2;
				}

			// Hacks are optional
			} else if (file_exists($filepath)) {
				include_once $filepath;
			}

		// Forward to error output and stop if necessary
		} else {
			require $p['core'].'error.php';
			break 2;
		}

	}

}



// Output & cache
// FLAG where should I set this?
date_default_timezone_set($settings['timezone']);

// Forward to a file
if ($response['file output'] == 'redirect') {
	header('Location: '.$output);

// Read a file for output
} else if ($response['file output'] == 'read file') {

	// Set headers
	foreach ($headers as $string) {
		header($string);
	}

	// Read and output file
	$file = fopen($output, 'rb');
	while (!feof($file)) {
		echo fread($file, 4096);
		ob_flush();
		flush();
	}
	fclose($file);



// Output is generated
} else {

	// Cache

	// Save the output in cache if allowed
	if ($settings['cache'] > 0 and isset($action['cache']) and $action['cache'] > 0) {

		// Cache location
		$directory = $p['cache'].$app['id'].'/'.$action['parent'].'/'.$action['id'].'/';

		// Create directory if it doesn't exist
		if (!is_dir($directory)) {
			mkdir($directory, 0777, true);
		}

		// Cache file's full path and generated filename
		$filepath = $directory.cachename_encode($input, $response['content type']);

		// Remove existing cache entry
		// FLAG need to test if this is necessary or if writing will always override existing file
		if (file_exists($filepath)) {
			unlink($filepath);
		}

		// Generated image
		if ($response['content type'] == 'image/jpeg') {
			imagejpeg($output, $filepath, 95);
		} else if ($response['content type'] == 'image/png') {
			imagealphablending($output, false);
			imagesavealpha($output, true);
			imagepng($output, $filepath, 0);
		} else if ($response['content type'] == 'image/gif') {
			imagegif($output, $filepath, 95);

		// Normal file
		} else {
			file_put_contents($filepath, $output);
		}

	}



	// Output the generated response

	// Set headers
	foreach ($headers as $string) {
		header($string);
	}

	// Output image
	if ($response['content type'] == 'image/jpeg') {
		imagejpeg($output, NULL, 95);
	} else if ($response['content type'] == 'image/png') {
		imagealphablending($output, false);
		imagesavealpha($output, true);
		imagepng($output, NULL, 0);
	} else if ($response['content type'] == 'image/gif') {
		imagegif($output, NULL);

	// Print text
	} else {
		echo $output;
	}

}
?>