<?php
// A couple of handy image manipulation scripts
// Jerry Jäppinen
// eiskis@gmail.com
// http://eiskis.net/
// WTFPL'd

/*
 * DO WHAT THE FUCK YOU WANT TO PUBLIC LICENSE
 * TERMS AND CONDITIONS FOR COPYING, DISTRIBUTION AND MODIFICATION
 *
 * 0. You just DO WHAT THE FUCK YOU WANT TO.
 */



// Does the math for changing image's dimensions
// 
// ACCEPTS
// 		1. Image file
// 		2. Desired width
// 		3. Desired height
// 		4. Desired area of the original image as a percentage
// 			100:		The whole image, no cropping (default)
// 			0:			Image is not scaled, the image only crops to the desired dimensions
// 
// RETURNS
// 		1. Original width
// 		2. Original height
// 		3. Result width
// 		4. Result height
// 		5. Crop area width
// 		6. Crop area height
function images_dimensions($file, $x = 0, $y = 0, $crop = -1) {

	// Dimensions of the original file
	$dimensions = getimagesize($file);
	
	// Original and desired width and height
	$return['original_x'] = $dimensions[0];
	$return['original_y'] = $dimensions[1];
	$return['destination_x'] = $x;
	$return['destination_y'] = $y;
	
	// Calculate the aspect ratio of the original image
	$ratio = $return['original_x']/$return['original_y'];
	
	// Default to original dimensions
	if ($return['destination_x'] == 0 and $return['destination_y'] == 0) {
		$return['destination_x'] = $return['original_x'];
		$return['destination_y'] = $return['original_y'];

	// Only desired height provided
	} elseif ($return['destination_x'] <= 0) {
		$return['destination_x'] += ($return['destination_y']+abs($return['destination_x']))*$ratio;

	// Only desired width provided
	} elseif ($return['destination_y'] <= 0) {
		$return['destination_y'] += ($return['destination_x']+abs($return['destination_y']))/$ratio;

	// All desired dimension given, keeping original aspect ratio (we have a bounding box!)
	} elseif ($return['destination_y']*$ratio < $return['destination_x'] and $crop === -1) {
		$return['destination_x'] = $return['destination_y']*$ratio;
	} elseif ($return['destination_x']/$ratio < $return['destination_y'] and $crop === -1) {
		$return['destination_y'] = $return['destination_x']/$ratio;
	}
	
	// Desired dimension(s) were bigger than in the original, need to scale down
	if ($return['destination_x'] > $return['original_x'] or $return['destination_y'] > $return['original_y']) {
		if ($return['destination_x'] > $return['original_x']) {
			$return['destination_x'] = $return['original_x'];
		}
		if ($return['destination_y'] > $return['original_y']) {
			$return['destination_y'] = $return['original_y'];
		}
		if ($return['destination_x'] > $return['destination_y']) {
			$return['destination_x'] = $return['destination_y']*$ratio;
		} else {
			$return['destination_y'] = $return['destination_x']/$ratio;
		}
	}
	
	// Figure out the size of the croppable area in the original image
	if ($crop >= 0) {

		// Make sure the relative size of the croppable area is sensible
		if ($crop >= 100) {
			$crop = 100;
		} else if ($crop > 0) {
			$crop = ceil($crop);
		}

		// Cropping as much as possible (no scaling of original image)
		if ($crop == 100) {
			$return['crop_x'] = $return['original_x'];
			$return['crop_y'] = $return['original_y'];

		// Turn a relative size of the croppable area into absolute dimensions
		} else {
			
			// Minimum cropping (keeping as much original image visible as possible)
			// Keeping the whole height
			if ($ratio > $return['destination_x']/$return['destination_y']) {
				$return['crop_y'] = $return['destination_y'];
				$return['crop_x'] = $return['destination_y']*$ratio;

			// Keeping the whole width
			} else {
				$return['crop_x'] = $return['destination_x'];
				$return['crop_y'] = $return['destination_x']/$ratio;
			}

			// Increasing the croppable area size
			if ($crop > 0) {

				$cropratio = $crop/100;
				$temp['x'] = $return['original_x'] - $return['destination_x'];
				$temp['y'] = $return['original_y'] - $return['destination_y'];

				$return['crop_x'] += ceil($temp['x']*$cropratio);
				$return['crop_y'] += ceil($temp['y']*$cropratio);
			
			}

		}
		
	}
	
	// Final round-ups to dimensions
	foreach ($return as $key => $value) {
		$return[$key] = ceil($value);
	}
	
	return $return;
	
}

// Scales/crops and returns a JPG, GIF or PNG
// 
// file			Image file
// x			Desired width
// y			Desired height
// crop			Desired area of the original image as a percentage
// 		100		The whole image, no cropping (default)
// 		0		Image is not scaled, the image only crops to the desired dimensions
function images_scale($file, $x = 0, $y = 0, $crop = -1) {

	if (is_file($file)) {
		
		$type = detect($file, 'extension');

		// Haetaan kuvien mitat
		$dimensions = images_dimensions($file, $x, $y, $crop);

		// Luodaan uusi kuva alkuperäisestä tiedostosta
		switch ($type) {
			case 'jpg':
				$image = imagecreatefromjpeg($file);
				break;
			case 'png':
				$image = imagecreatefrompng($file);
				break;
			case 'gif':
				$image = imagecreatefromgif($file);
				break;
		}
		
		// Luodaan palautettava kuva
		$return = imagecreatetruecolor($dimensions['destination_x'], $dimensions['destination_y']);
		imagefill($return, 0, 0, imagecolorallocatealpha($return, 255, 255, 255, ($type == 'png' ? 127 : 0)));
		
		// Leikataan kuvasta
		if (isset($dimensions['crop_x']) and isset($dimensions['crop_y'])) {
			
			$point_x = ceil(($dimensions['crop_x']-$dimensions['destination_x'])/2);
			$point_y = ceil(($dimensions['crop_y']-$dimensions['destination_y'])/2);
			
			if ($crop == 100) {
				imagecopy($return, $image, 0, 0, $point_x, $point_y, $dimensions['crop_x'], $dimensions['crop_y']);
			} else {
				imagecopyresampled($return, $image, -$point_x, -$point_y, 0, 0, $dimensions['crop_x'], $dimensions['crop_y'], $dimensions['original_x'], $dimensions['original_y']);
			}
			
		// Pienennetään
		} else {
			imagecopyresampled($return, $image, 0, 0, 0, 0, $dimensions['destination_x'], $dimensions['destination_y'], $dimensions['original_x'], $dimensions['original_y']);
		}
		
		// Poistetaan lähdekuva muistista
		imagedestroy($image);
		
		return $return;

	} else {

		return false;

	}

}

?>