<?php
// Parsing Proot-style app packages
// Jerry Jäppinen
// eiskis@gmail.com
// http://eiskis.net/
// WTFPL'd

/*
 * DO WHAT THE FUCK YOU WANT TO PUBLIC LICENSE
 * TERMS AND CONDITIONS FOR COPYING, DISTRIBUTION AND MODIFICATION
 *
 * 0. You just DO WHAT THE FUCK YOU WANT TO.
 */



// Get package contents for an app
function package_map_contents ($app) {

	global $p;
	global $constants;

	$result = array();
	$accepted_formats = array_flatten($constants['app package']['file types']);

	// Extract tools based on configuration
	$temp = package_map_tools($app['tools'], $p['tools'], $accepted_formats);
	$result['tools'] = array(
		'external' => $temp['absolute'],
		'internal' => $temp['relative']
	);

	// Add trimmed paths of included tools
	$temp = $p['apps'].$app['id'].'/'.$constants['app package']['tools'];
	foreach (rglob_files($temp, $accepted_formats) as $value) {
		$result['tools']['included'][] = substr($value, strlen($temp));
	}

	// Add trimmed paths of shared code
	$temp = $p['apps'].$app['id'].'/'.$constants['app package']['shared'];
	foreach (rglob_files($temp, $accepted_formats) as $value) {
		$result['shared'][] = substr($value, strlen($temp));
	}

	// App's views
	$path = $p['apps'].$app['id'].'/'.$constants['app package']['views'];
	if (is_dir($path)) {
		$result['views'] = package_map_views($path);
	}

	return $result;
}



// Fetch a valid file listing based on a manifest
function package_map_tools ($manifest = array(), $root = '', $formats = array()) {

	global $constants;

	$result = array('relative' => array(), 'absolute' => array());

	// Make sure we have something to work with
	if (!empty($manifest) and !empty($root) and !empty($formats)) {
		foreach (to_array($manifest) as $path) {

			// Files
			if (in_array(detect($path, 'extension'), $formats)) {

				// External URLs
				if (preg_match($constants['patterns']['url'], $path)) {
					$result['absolute'][] = $path;

				// Internal files with acceptable file types
				} else if (is_file($root.$path)) {
					$result['relative'][] = $path;
				}

			// Directories
			} else if (is_dir($root.$path)) {

				// Get contents recursively
				$temp = rglob_files($root.$path, $formats);
				foreach ($temp as $value2) {

					// FLAG this replacing might break, should only get stuff from beginning of path
					$result['relative'][] = str_replace($root, '', $value2);
				}

			}

		}
	}

	return $result;
}



// Extract asset files of an app package into a map of assets
// 
// Result format
// 		child => 
// 			behavior => ['init.js', 'model.json'],
// 			structure => ['foo.html'],
// 			style => ['bar.css'],
// 			children => 
// 				child =>
// 				child =>
function package_map_views ($path, $level = 1, $maximum_levels = 5) {
	
	global $constants;

	$return = array();

	foreach (glob($path.'*', GLOB_MARK) as $value) {
		
		// Recursive folders
		if (is_dir($value) and $level <= $maximum_levels) {
			$temp = pathinfo($value, PATHINFO_FILENAME);
			$return['children'][$temp] = package_map_views($value, $level + 1);
			if (empty($return['children'][$temp])) {
				unset($return['children'][$temp]);
			}

		// Sort files into categories
		} else {
			
			foreach ($constants['app package']['file types'] as $key2 => $value2) {
				if (in_array(detect($value, 'extension'), $value2)) {
					$return[$key2][] = basename($value);
					break;
				}
			}

		}
		
	}

	if (empty($return['children'])) {
		unset($return['children']);
	}
	
	return $return;

}
?>
