<?php
// Helper methods for Proot

// Whether or not using shared database
function is_shared_database () {
	global $settings;
	return !empty($settings['database']['database']);
}

// Prefix table names when using shared database name
function object_to_table ($name) {
	global $app;
	global $settings;

	if (is_shared_database()) {
		$name = str_replace('_', '', $settings['database']['prefix']).$app['id'].$name;
	}

	return $name;
}

// Unprefix a table name appropriately
function table_to_object ($name) {
	global $app;
	global $settings;

	debug($name);
	if (is_shared_database()) {
		$name = substr($name, strlen(str_replace('_', '', $settings['database']['prefix']).$app['id']));
	}

	debug($name);
	return $name;
}

// Connect to database
function app_to_database ($app_id) {
	global $settings;

	// Shared database
	if (is_shared_database()) {
		return connect_to_database($settings['database']['database']);

	// App-specific database
	} else {
		return connect_to_database((empty($settings['database']['prefix']) ? '': dont_end_with($settings['database']['prefix'], '_').'_').'app_'.$app_id);
	}
}

// Connect to a database
function connect_to_database ($database) {
	global $settings;

	// Different DB types need different setup credentials in different format
	if ($settings['database']['type'] == 'sqlite') {
		$string = $settings['database']['type'].':/tmp/'.$database.'.txt';
	} else {
		$string = $settings['database']['type'].':host='.$settings['database']['address'].';'.($settings['database']['type'] == 'cubrid' ? 'port:3000;' : '').'dbname='.$database;
	}

	// Get the connection
	try {
		$connection = R::setup($string, $settings['database']['username'], $settings['database']['password']);
		R::$adapter->getDatabase()->connect();

	// Connection failed
	} catch (PDOException $e) {
		R::close();
		$connection = false;
	}

	return $connection;
}
?>