
// Exposed backend functionality
p.actions = {

	// Provide some schematic info
	map: function () {
		return app.actions;
	},

	// Output a URL that targets an action
	link: function (parent, action, inputs) {
		var app = p.app.get();
		var input = '';

		var parameters = bl.trim.empties([app.id, parent, action]);
		parameters = parameters.join('/') + '/';

		if (inputs !== undefined) {

			// Allow passing inputs as a single value
			if (bl.is.str(inputs) || bl.is.int(inputs)) {
				inputs = [inputs];

			// Pick values from hash
			} else if (bl.is.hash(inputs)) {
				inputs = bl.to.array(inputs);
			}
			
			// No parameters to pass
			if (inputs.length) {

				// Encode and validate inputs
				for (var i = 0; i < inputs.length; i++) {
					if (!bl.is.empty(inputs[i])) {
						var newInput = '' + inputs[i];

						// Double escape slashes for the server
						// FLAG should replace all slashes with one regexp, propably
						newInput = newInput.replace(new RegExp(p.text.to.pattern('/'), 'g'), '%252F');
						newInput = newInput.replace(new RegExp(p.text.to.pattern('\\'), 'g'), '%255C');

						input += newInput + '/';
					}
				}

			}

		}
		
		return app.paths.use + parameters + input;
	},

	// Request an action with GET
	// FLAG this doesn't actually return the data: add support for callback functions
	get: function (parent, action, input) {
		var url = p.actions.link(parent, action, input);
		return $.get(url);
	},

	// Request an action with POST
	post: function (parent, action, input) {
		return $.post(p.actions.link(parent, action), bl.to.hash(input));
	},

	// Execute an action with arbitrary parameters (handy for creating shorthands)
	// NOTE: doesn't allow naming parameters (i.e. passing them in a hash)
	//		We could do that if we could know when the first parameter happens to be a hash
	contact: function (parent, action, parameters) {
		return p.actions.post(parent, action, parameters);
	}

};
