
// Control spacetime and that what exists
p.control = {

	// Return a promise object after the duration
	wait: function (duration) {
		var deferred = $.Deferred();

		// Resolve immediately on empty
		if (bl.is.empty(duration)) {
			deferred.resolve();

		// Return after set timeout
		} else {
			setTimeout(function () {

				// Reject or resolve
				if (duration < 0) {
					deferred.reject();
				} else {
					deferred.resolve();
				}

			}, Math.abs(duration));

		}

		return deferred.promise();
	},

	// Launch a method every n milliseconds
	// NOTE
	//    is reported done when the last interval STARTS
	//    never rejects
	// FLAG allow passing in arguments, use apply to run callback with them
	periods: function (count, duration, callback) {
		var deferred = $.Deferred();
		
		// Normalize to positive values
		duration = Math.abs(duration);

		// Run for all the iterations
		for (var i = 0; i < count; i++) {

			(function () {
				var j = i;

				// Wait an appropriate interval
				p.control.wait(duration*j).then(function () {
					callback();
					if (j == count-1) {
						deferred.resolve();
					}
				});

			})();

		}
		return deferred.promise();
	},

	// Run a callback method n times, with a set interval between a FINISHED iteration and the START of the next
	// NOTE
	//    callback must return a promise object
	//    resolves itself after the last callback resolves
	//    rejects immediately if any of the callbacks rejects
	// FLAG allow passing in arguments, use apply to run callback with them
	intervals: function (count, interval, callback) {

		// Run callback after waiting
		var _run = function (iterations, func) {
			var dfd = $.Deferred();

			// Run, if there are iterations left
			if (iterations > 0) {
				p.control.wait(interval).then(function () {
					func().then(function () {
						_run(iterations-1, func).then(function () {
							dfd.resolve();
						}, function () {
							dfd.reject();
						});
					});
				});

			// No iterations left, we're done
			} else {
				dfd.resolve();
			}

			return dfd.promise();
		};

		var deferred = $.Deferred();

		// Normalize to positive values
		interval = Math.abs(interval);

		// Run callback once immediately, then wait before each future iteration
		callback().then(function () {
			_run(count-1, callback).then(function () {
				deferred.resolve();
			}, function () {
				deferred.reject();
			});
		});

		return deferred.promise();
	}

};
