
// DOM normalization and manipulation
// FLAG: I must decide where and how I want to normalize the treatment of multiple or just one element (since jQuery selectors tend to select multiple elements)
p.dom = {

	types: {

		// Show a map of all configured DOM element types and their respective selectors
		map: function () {
			return bl.get.value(p.conf.dom.types);
		},

		// List available types
		list: function () {
			return bl.list.keys(p.dom.types.map());
		}

	},



	// Yes or no -_-
	is: {

		// Check if a DOM element is disabled
		disabled: function (element) {
			return element.hasClass('disabled');
		},
		enabled: function (element) {
			return !p.dom.is.disabled(element);
		},

		// Visibility
		hidden: function (element) {
			return element.hasClass('hidden') || element.is(':hidden');
		},
		visible: function (element) {
			return !p.dom.is.hidden();
		}

	},

	to: {

		// Toggle disabled/enabled state of an DOM element
		// FLAG minor repetition
		disabled: function (element) {
			var type = p.dom.detect(element);
			element.addClass('disabled');
			if (p.dom[type] && p.dom[type].to) {
				return p.dom[type].to.disabled(element);
			}
		},
		enabled: function (element) {
			var type = p.dom.detect(element);
			element.removeClass('disabled');
			if (p.dom[type] && p.dom[type].to) {
				return p.dom[type].to.enabled(element);
			}
		},
		hidden: function (element) {
			var type = p.dom.detect(element);
			element.addClass('hidden');
			if (p.dom[type] && p.dom[type].to) {
				return p.dom[type].to.hidden(element);
			}
		},
		visible: function (element) {
			var type = p.dom.detect(element);
			element.removeClass('hidden');
			if (p.dom[type] && p.dom[type].to) {
				return p.dom[type].to.visible(element);
			}
		}

	},



	// Detect the type of a DOM element
	detect: function (element) {
		var types = p.dom.types.map();
		for (var key in types) {
			if (element.is(types[key])) {
				return key;
			}
		}
	},

	// Interpret and normalize the current value of a DOM element
	extract: function (element) {
		var type = p.dom.detect(element);
		if (bl.is.set(p.dom[type])) {
			return p.dom[type].extract(element);
		}
	},

	// Select all items of a specific type in a container
	select: function (type, container) {

		var result = $('', container);
		var selectors = [];
		var conf = p.dom.types.map();
	
		// Support giving multiple selectors
		var types = bl.to.array(type);
		
		// Fallback to app's wrapper
		if (!bl.is.set(container)) {
			container = p.app.get().wrapper;
		}

		// Get selectors from config
		for (var i = 0; i < types.length; i++) {
			if (bl.is.set(conf[types[i]])) {
				selectors.push(conf[types[i]]);
			} else {
				selectors.push(types[i]);
			}
		}

		result = $(selectors.join(', '), container);

		return result;
	},



	// Insert content into the selected DOM element (defer to type-specific method)
	populate: function (element) {
		var type = p.dom.detect(element);
		if (bl.is.set(p.dom[type])) {
			return p.dom[type].populate.apply(p.dom[type], arguments);
		}
	},

	// Replace placeholders in UI item template with content
	// FLAG bad name
	fill: function (template, content) {

		var result = bl.get.value(template);

		// Assuming a DOM object if template is not string
		if (!bl.is.str(result)) {
			result = p.html.raw(result);
		}

		// Named keys for content
		if (bl.is.hash(content)) {
			for (var key in content) {
				result = result.replace(new RegExp(p.text.to.pattern('$' + key), 'g'), content[key]);
			}

		// Just replace in order
		} else {
			content = bl.to.array(content);
			for (var i = 0; i < content.length; i++) {
				result = result.replace(new RegExp(p.text.to.pattern('$' + (i+1)), 'g'), content[i]);
			}
		}
		
		return result;
	},



	// Convert a collection of selectors into a collection of jQuery objects
	convert: function (selectors, container) {
		var result = bl.to.empty(selectors);

		// Fallback to app's wrapper
		if (!bl.is.set(container)) {
			container = p.app.get().wrapper;
		}

		if (!bl.is.empty(selectors)) {
			for (var key in selectors) {
				var value = selectors[key];

				// Collections get to take another round
				if (bl.is.hash(value) || bl.is.array(value)) {
					result[key] = p.dom.convert(value);

				// Others get converted
				} else if (bl.is.array(result)) {
					result.push($(value, container));
				} else if (bl.is.hash(result)) {
					result[key] = $(value, container);
				}

			}
		}

		return result;
	},

	// Populate a view container with content
	// Content is JSON as key-value pairs with keys that match the model
	// NOTE doesn't handle multiple pieces of content (we could use arrays but that would be too elaborate)
	distribute: function (model, content) {

		if (model && content) {
			for (var key in model) {
				var item = model[key];
				if (bl.is.hash(item) && bl.is.hash(content[key])) {
					p.dom.distribute(item, content[key]);
				} else if (bl.is.set(content[key])) {
					p.dom.populate(item, content[key]);
				}
			}
			return true;
		}
		
		return false;
	}

};
