
// State handling
// Requires plugins/jquery.history.js
//
// NOTE
// What we have:
// appState: hash of state information
//		title:	""				an arbitrary name for the state
//		url:	""				associated URL pointer
//		views:	[				a list of views
//						{
//							type:
//							view:
//							wrapper:
//							content
//						}
//					]
p.state = {

	// Initialize state support
	initialize: function () {

		// Bind History state changes to custom behavior
		//
		// NOTE
		//	- Browser-initiated navigation triggers ONLY this
		//	- App-initiated navigation runs functions that ensure a valid state and then trigger this via History
		window.History.Adapter.bind(window, 'statechange', function () {

			// FLAG I should make sure that, after updating, the actual state reflects History state
			p.state.update();

		});
	},

	// Detect if the state system is supported in UA
	supported: function () {
		if (window.History) {
			return window.History.enabled;
		} else {
			return false;
		}
	},



	// Get state info
	current: function () {
		return bl.get.value(app.state);
	},
	previous: function () {
		var previousState = {};
		if (History.savedStates.length > 1) {
			previousState = History.savedStates[History.savedStates.length-2].data;
		}
		return previousState;
	},
	list: function () {
		var states = [];
		for (var i = 0; i < History.savedStates.length; i++) {
			states.push(History.savedStates[i].data);
		}
		return states;
	},



	// Update app to have the current state's views rendered
	// FLAG
	//    duplicate logic with fetch
	//    stupid repetition
	//    doesn't return a promise object
	//    calls rendering functions on empty view sets
	update: function () {

		var historyState = History.getState();

		// Update running variables
		window.app.state = historyState.data;
		var app = p.app.get();

		// Update app's internal URL info
		// FLAG do I want to rely on app.url, when I can use p.paths to fetch current info?
		var parameters = p.paths.parameters();
		for (var i = 0; i < app.url.length; i++) {
			if (parameters[i]) {
				window.app.url[i] = parameters[i];
			} else {
				window.app.url[i] = '';
			}
		}

		// We can create, update or dispose of views
		var views = {
			create: [],
			update: [],
			dispose: []
		};

		// Some views are already there
		if (History.savedStates.length > 1) {
			var previousState = History.savedStates[History.savedStates.length-2].data;
			views = p.state.views.changes(previousState.views, app.state.views);

		// First time, create everything
		} else {
			views.create = app.state.views;
		}

		// Dispose of old views
		if (!bl.is.empty(views.dispose)) {
			p.state.views.run('dispose', views.dispose).then(function () {

				// Create new views
				$.when(p.state.views.run('update', views.update), p.state.views.run('fetch', views.create)).then(function () {

				}, function () {
					return false;
				});

			}, function () {
				return false;
			});

		// Nothing to dispose
		} else {

			// Create new views
			$.when(p.state.views.run('update', views.update), p.state.views.run('fetch', views.create)).then(function () {

			}, function () {
				return false;
			});

		}

		return true;
	},

	

	// Get notified when all transitions are finished
	ready: function (interval) {
		var deferred = $.Deferred();

		// Tells if app has any transitions going on
		var _ready = function () {
			return (p.app.get().transitions === 0 ? true : false);
		};

		// Ensure proper interval is set
		if (!interval) {
			interval = bl.get.value(p.conf.pollInterval);
		} else {
			interval = bl.to.int(interval);
		}

		// App's ready, no reason to wait
		if (_ready()) {
			deferred.resolve();

		// Poll for app status periodically
		} else {
			var poll = setInterval(function () {
				if (_ready()) {
					poll = clearInterval(poll);
					deferred.resolve();
				}
			}, interval);
		}
		
		return deferred.promise();
	}
	
};
