
// Trigger state change via History
p.state.to = {

	// Navigate to a complete state
	state: function (newState, replace) {
		var deferred = $.Deferred();

		p.state.ready().then(function () {

			// Use the correct prefixes in URL
			if (newState.url && newState.url != p.state.current().url) {
				newState.url = p.paths.get.app() + newState.url;
			}

			// New title
			if (!newState.title) {
				newState.title = p.state.current().title;
			}

			// Replace current state
			// FLAG I should check for a duplicate state
			//		replace (if there were minor changes)
			//		ignore (if the states were an exact match)
			//		but is this the place? I should enforce that in state handling, not user methods
			if (replace) {

				// Remove first state from history on initial page load
				// FLAG is this necessary?
				if (History.savedStates.length == 1) {
					History.savedStates.pop();
					History.storedStates.pop();
				}

				History.replaceState(newState, newState.title, newState.url);

			// Add a completely new state
			} else {
				History.pushState(newState, newState.title, newState.url);
			}

			// Poll the state of the app until all transitions are complete
			p.state.ready().then(function () {
				deferred.resolve();
			});

		}, function () {
			deferred.reject();
		});

		return deferred.promise();
	},

	// Navigate between stored states
	previous: function () {
		if (p.app.get().transitions === 0) {
			return History.back();
		} else {
			return false;
		}
	},
	next: function () {
		if (p.app.get().transitions === 0) {
			return History.forward();
		} else {
			return false;
		}
	},



	// Shorthands to p.state.to.state()
	// NOTE p.state.to.state() assumes a valid state, these shorthands do validation and normalization based on what kind of input they take

	// Navigating to a new, complete state, for the first time
	// FLAG this is redundant
	start: function (views) {

		var app = p.app.get();

		var state = {
			title: app.meta.name,
			views: []
		};

		// Allow shorthand input
		if (bl.is.hash(views)) {
			state.views = [views];
		} else if (bl.is.array(views)) {
			state.views = views;
		}

		return p.state.to.state(state, true);
	},

	// Add the desired view to state
	view: function (view, url, title, replace) {
		var newState = p.state.current();

		// Keep all old views that don't have overlapping wrappers
		// FLAG doesn't look for child views, so those aren't disposed of properly even if parent is removed
		//		internal state handling should automatically eradicate those, I guess
		newState.views = bl.remove.keys(newState.views, p.state.views.where(newState.views, {
			'wrapper': view.wrapper
		}));

		// Add the desired view into the state
		newState.views.push(view);

		// Optionally update URL and title
		newState.url = undefined;
		newState.title = undefined;
		if (url) {
			newState.url = url;
		}
		if (title) {
			newState.title = title;
		}

		return p.state.to.state(newState, replace);
	}

};
