
// Views within a state, i.e. handling multiple views at once
// FLAG these remain somewhat untested, and not utilized by internal state system
p.state.views = {

	// List views
	current: function () {
		return p.state.current().views;
	},
	previous: function () {
		var previousState = p.state.previous();
		return bl.is.empty(previousState) ? [] : previousState.views;
	},

	// Search for certain types of views from current views
	// conditions = {key: value};
	// FLAG this should use self.compare()
	where: function (views, conditions) {
		var numberOfChecks = bl.get.length(conditions);
		var keys = bl.get.value(p.conf.state);
		var indexes = [];

		// Allow giving conditions as a shorthand
		if (bl.is.array(conditions)) {
			conditions = bl.to.hash(conditions, keys);
		}

		// Match criteria against all the views
		for (var i = 0; i < views.length; i++) {
			var view = views[i];

			// Conditions are type-value pairs
			var j = 0;
			for (var key in conditions) {

				// Passes the check
				if (view[key] === conditions[key]) {
					j++;

					// All checks were passed
					if (j == numberOfChecks) {
						indexes.push(i);
					}

				// Failed the test
				} else {
					break;
				}

			}

		}

		return indexes;
	},



	// Understanding views in state

	// Check if two views have the same attributes
	compare: function (view, viewToCompare, conditions) {
		var result = false;
		var keys = ['type', 'view', 'wrapper', 'content'];

		// Default to the strictest comparison
		if (!conditions) {
			conditions = keys;
		} else {
			conditions = bl.to.array(conditions);
		}

		if (view && viewToCompare) {
			result = true;
			for (var i = 0; i < conditions.length; i++) {
				var key = conditions[i];
				if (!bl.are.equal(view[key], viewToCompare[key])) {
					result = false;
					break;
				}
			}
		}

		return result;
	},

	// Compare two sets of views to find out their differences
	changes: function (oldSet, newSet) {

		var i, j, oldView, newView;
		var conditions = bl.remove.values(['content'], bl.get.value(p.conf.state));

		var result = {
			create: [],
			dispose: [],
			update: []
		};

		// Take a look at the old views
		for (i = 0; i < oldSet.length; i++) {
			oldView = oldSet[i];

			// Assume we need to dispose of the old view
			result.dispose.push(oldView);

			// Compare the old view to each new view
			for (j = 0; j < newSet.length; j++) {
				newView = newSet[j];

				// But if it's in the new set, we can keep it
				if (p.state.views.compare(oldView, newView, conditions)) {
					result.dispose.pop();
					break;
				}

			}

		}

		// Take a look at the new views
		for (i = 0; i < newSet.length; i++) {
			newView = newSet[i];

			// Assume we need to create the new view
			result.create.push(newView);

			// Compare the new view to each old view, since we might have duplicates
			// FLAG some have been marked for disposal already, no need to compare to those
			for (j = 0; j < oldSet.length; j++) {
				oldView = oldSet[j];

				// The new view is already there
				if (p.state.views.compare(newView, oldView, conditions)) {

					// Remove from creatables
					result.create.pop();

					// But still update if content changed
					if (!bl.are.equal(newView.content, oldView.content)) {
						result.update.push(newView);
					}

				}

			}

		}

		return result;
	},



	// Execute view handling for multiple views synchronously
	// FLAG doesn't do it synchronously -___________________-
	// NOTE
	//    Views override others when they are loaded into the same container
	//    Fetches all the views into their containers (when we always use the same containers, things will naturally work out)
	run: function (method, views) {
		var deferred = $.Deferred();
		var app = p.app.get();

		// Some views were requested
		if (method && views.length) {

			// Just render the changes
			var i, j;
			for (i = 0, j = 0, amount = views.length; i < amount; i++) {
				var view = views[i];

				// Restrict container to within app
				var wrapper = $(view.wrapper, '.' + p.conf.views.classNames.app + '_' + app.id);
				if (wrapper.length) {

					// Mark transition ongoing
					window.app.transitions++;

					// Execute the requested view method correctly
					var callable = p.views.life[method];
					var self = p.views.life;
					var parameters;

					// We need to call different things for different events
					// FLAG this is pretty retarded, we should not have to do this
					if (method == 'fetch') {
						parameters = [wrapper, view.type, view.view, view.content];
					} else if (method == 'dispose') {
						parameters = [wrapper, view.content];
					} else if (method == 'update') {
						callable = p.views.life.run;
						parameters = [wrapper, 'update', 'default', view.content];
					}

					// Call the lifetime method
					callable.apply(self, parameters).then(function () {

						// Transition complete
						window.app.transitions--;

						// Final view is done
						if (j == amount - 1) {
							deferred.resolve();
						}

						j++;

					// Lifetime method failed
					}, function () {
						// console.log([method] + ' failed for:');
						// console.log(view);
						window.app.transitions--;
						deferred.reject();
						j++;

					});

				}
			}
			
		// No changes needed
		} else {
			deferred.resolve();
		}

		return deferred.promise();
	}

};
