
// Parse strings of specific format
p.text.parse = {

	// Dig for key-value pairs
	pairs: function (string, pairSeparator, valueSeparator) {
		var result = {};
		pairs = string.split(pairSeparator);
		for (var i = 0; i < pairs.length; i++) {
			var pair = pairs[i].split(valueSeparator);
			result[pair[0]] = pair[1];
		}
		return result;
	},

	// Split an e-mail address into pieces
	email: function (string) {
		var parts = string.replace('mailto:', '').split('@');
		return {
			'user': parts[0],
			'domain': parts[1]
		};
	},

	// Parse key-value pairs from a GET parameter string (FLAG not checking for "?")
	get: function (string) {
		return p.text.parse.pairs(string.replace('?', ''), '&', '=');
	},

	// Parse shorthand string format (used in input URLs)
	shorthand: function (string) {
		return p.text.parse.pairs(string, ';', ':');
	},

	// Splits URL to pieces
	// FLAG
	//    trailing slashes aren't normalized
	//    doesn't work when there's no protocol
	url: function (string) {

		var expression = new RegExp(
			'^' +
			'(?:' +
			'([^:/?#.]+)' +						// protocol (ignore special characters used by other URL parts such as :, ?, /, #, and .)
			':)?' +
			'(?://' +
			'(?:([^/?#]*)@)?' +					// username
			'([\\w\\d\\-\\u0100-\\uffff.%]*)' +	// domain (restrict to letters, digits, dashes, dots, percent escapes, and unicode characters)
			'(?::([0-9]+))?' +					// port
			')?' +
			'([^?#]+)?' +						// path
			'(?:\\?([^#]*))?' +					// query
			'(?:#(.*))?' +						// fragment
			'$');

		var parts = string.match(expression);

		// Get parameters as a hash
		if (parts[6]) {
			parts[6] = p.text.parse.get(parts[6]);
		}

		return {
			'protocol':parts[1],
			'user':parts[2],
			'domain':parts[3],
			'port':parts[4],
			'path':parts[5],
			'parameters': parts[6],
			'fragment':parts[7]
		};

	}

};
