
// Viewport dimensions, detection and monitoring
p.viewport = {

	// Run all viewport functions, get values in one object
	report: function (subset) {
		var result = {};

		// Choose what to include in report
		var keys;
		if (arguments.length) {
			keys = arguments;
		} else {
			keys = bl.list.keys(bl.remove.key(p.viewport, 'report'));
		}

		// Run everything to include in report
		for (var i = 0; i < keys.length; i++) {
			result[keys[i]] = {};
			for (var func in p.viewport[keys[i]]) {
				result[keys[i]][func] = p.viewport[keys[i]][func]();
			}
		}

		return bl.get.length(result) === 1 ? bl.find.first.value(result) : result;
	},



	// Dimensions
	dimensions: {
		width: function () {
			return document.documentElement.clientWidth;
		},
		height: function () {
			return document.documentElement.clientHeight;
		}
	},

	// Position of viewport relative to canvas
	// NOTE negative values never considered valid
	position: {

		top: function () {
			var offset = $(window).scrollTop();
			return offset > 0 ? offset : 0;
		},
		right: function () {
			return p.viewport.position.left() + p.viewport.dimensions.width();
		},
		bottom: function () {
			return p.viewport.position.top() + p.viewport.dimensions.height();
		},
		left: function () {
			var offset = $(window).scrollLeft();
			return offset > 0 ? offset : 0;
		}
		
	},

	// Body (also app.container)
	// FLAG broken as hell
	canvas: {

		width: function () {
			return document.body.clientWidth;
		},
		height: function () {
			return document.body.clientHeight;
		}

	},

	// Amount of overflow screenspace on each side of screen
	overflow: {

		top: function () {
			return p.viewport.position.top();
		},
		right: function () {
			return p.viewport.canvas.width() - (p.viewport.position.left() + p.viewport.dimensions.width());
		},
		bottom: function () {
			return p.viewport.canvas.height() - (p.viewport.position.top() + p.viewport.dimensions.height());
		},
		left: function () {
			return p.viewport.position.left();
		}

	}

};
