
// Handling of individual view's life

// Each view is created with a container. Wrapper is the DOM object that the view was created into.
// Wrapper can be anything, but will be generated specific classes. Container has a specific format.
// Wrapper has its status as a class. These view handling methods are always called to the wrapper.

// NOTE
// Fetching makes sure that all the required assets and content is available for view lifecycle to start
//   - This is dependent on how the asset pipeline works
//   - Similarly, clear is responsible for cleaning up everything after view's life has ended

// FLAG
// The status system is badly done, the logic is split all over, hardcoded
//		- Have a simple logic for updating view's status when its lifetime methods are run
//		- Have the English words for the statuses in config
//		- When there's a simple logic for the statuses, I don't have to use prefixed lifetime methods internally
// If a view is fetched within itself, things probably go FUBAR
//   - Preparations will stack up and malfunction? Not sure what will happen -_-
//   - This should probably be fixed or prevented?
// We could also
//   - Load all structure into memory like with the rest of asset pipeline, or
//   - Keep loaded view materials in DOM and not reload them fully if we come back
//     -> we could do this with other assets, too
p.views = {

	// List views per type
	map: function () {
		var types = p.app.get().life.children;
		var result = {};
		for (var key in types) {
			result[key] = bl.list.keys(types[key].children);
		}
		return result;
	},
	// NOTE returns view types
	list: function () {
		return bl.list.keys(p.views.map());
	},



	// Identify a view in DOM
	detect: function (element) {

		var returns = {};
		var classes = p._.getClasses(element);

		// Detect the wrapper
		if (bl.has.value(p.conf.views.classNames.wrapper, classes)) {

			// FLAG I should really learn regexp
			for (var i = 0; i < classes.length; i++) {
				var className = classes[i];
				if (className.substring(0, 8) == p.conf.views.classNames.wrapper + '_') {
					var parts = className.split('_');
					if (parts[2]) {
						returns.type = parts[1];
						returns.name = parts[2];
						break;
					} else if (parts[1]) {
						returns.type = parts[1];
					}
				}
			}

		// Detect the view
		// FLAG I should evaluate the class names properly, not rely on correct order
		} else if (bl.has.value('view', classes)) {
			returns.type = classes[1] === undefined ? '' : classes[1].substring(5);
			returns.name = classes[2] === undefined ? '' : classes[2].substring(returns.type.length + 6);
		} else {
			returns = false;
		}

		return returns;
	},

	// Get the contents of app package's data files (merged starting from global, view type's and view's scope)
	collect: function (collectable, type, view) {

		// Construct a collection based on view type and view attributes
		// FLAG not very elegant (should be recursive)
		var app = p.app.get();
		var result = {};

		// Global level
		if (bl.is.hash(app.life[collectable])) {
			$.extend(result, app.life[collectable]);
		}

		// Type-specific level
		if (bl.is.set(type)) {
			if (bl.is.hash(app.life.children[type]) && bl.is.hash(app.life.children[type][collectable])) {
				$.extend(result, app.life.children[type][collectable]);
			}

			// View-specific level
			if (bl.is.set(view)) {
				if (bl.is.hash(app.life.children[type].children[view]) && bl.is.hash(app.life.children[type].children[view][collectable])) {
					$.extend(result, app.life.children[type].children[view][collectable]);
				}

			}

		}

		return result;
	}

};
