
// Running view lifetime methods
p.views.life = {

	// A shorthand to deliver a view into wrapper (create and/or dispose)
	// FLAG should this be used internally at all?
	fetch: function (wrapper, type, view, content) {

		var deferred = $.Deferred();
		
		// Only proceed if view's not being processed
		if (p.views.status.get(wrapper)) {
			deferred.reject();
		} else {

			// Wrapper is not empty, let's handle that first
			// FLAG no idea what kind of emptiness I should actually check for
			if (wrapper.children().length > 0) {

				// Dispose and create
				// FLAG only disposing of the first view here
				//   -> make dispose() support properly destroying the children, too
				p.views.life.dispose(wrapper).then(function () {
					p.views.life.create(wrapper, type, view, content).then(function () {
						deferred.resolve();
					}, function () {
						deferred.reject();
					});
				}, function () {
					deferred.reject();
				});

			// Wrapper is already empty
			} else {

				// Create
				p.views.life.create(wrapper, type, view, content).then(function () {
					deferred.resolve();
				}, function () {
					deferred.reject();
				});
			}

		}

		return deferred;
	},



	// Wrappers for lifetime event shortcuts

	// Load a new view, along with its structure, into a wrapper DOM element
	create: function (wrapper, type, view, content) {

		var deferred = $.Deferred();

		// Only proceed if view's not being processed
		if (p.views.status.get(wrapper)) {
			deferred.reject();
		} else {

			// Start loading content
			p.actions.get(p.conf.server.templateSource.parent, p.conf.server.templateSource.action, ['view', type, view]).then(function (data) {

				// Enforce standardized classes for wrappers
				// FLAG I could probably use some generic function for (some of) this
				var className = p.conf.views.classNames.wrapper;
				var newClasses = className +
									' ' + className + '_' + type +
									' ' + className + '_' + type + '_' + view;
				var oldClasses = p._.getClasses(wrapper);

				// Preserve non-interfering classes
				for (var i = 0; i < oldClasses.length; i++) {
					if (oldClasses[i].substring(0, 7) != className) {
						newClasses += ' ' + oldClasses[i];
					}
				}
				wrapper.attr('class', newClasses);

				// Wrap view into a container
				// FLAG I could probably use some generic function for (some of) this (doing the same as above, differently)
				var containerClasses = [p.conf.views.classNames.view];
				containerClasses.push(containerClasses[0] + '_' + type);
				containerClasses.push(containerClasses[1] + '_' + view);
				wrapper.wrapInner('<div class="' + containerClasses.join(' ') + '"></div>');

				// Insert new content into wrapper
				wrapper.children().first().html(data);
			
				// Proceed with preparation
				// FLAG force hide & show wrapper's contents here?
				p.views.life.run(wrapper, 'prepare', 'default', content).then(function () {
					p.views.life.run(wrapper, 'render', 'default', content).then(function () {
						deferred.resolve();
					}, function () {
						deferred.reject();
					});
				}, function () {
					deferred.reject();
				});
				
			}, function() {
				deferred.reject();
			});

		}

		return deferred.promise();
	},

	// Kill a view and clear the wrapper
	dispose: function (wrapper, content) {

		var deferred = $.Deferred();

		// Only proceed if view's not being processed
		if (p.views.status.get(wrapper)) {
			deferred.reject();
		} else {

			// Unrender first
			// FLAG force hide wrapper's contents here?
			p.views.life.run(wrapper, 'unrender', 'reverse', content).then(function () {

				// Finally kill any views in the way
				p.views.life.run(wrapper, 'kill', 'reverse', content).then(function () {

					// Remove the view-specific class
					// FLAG should not use detect here, should use regexp
					var temp = p.views.detect(wrapper);
					var className = p.conf.views.classNames.wrapper + '_' + temp.type + '_' + temp.name;
					wrapper.removeClass(className);

					// Evaporate HTML
					wrapper.empty();

					deferred.resolve();
					
				}, function () {
					deferred.reject();
				});

			}, function () {
				deferred.reject();
			});

		}

		return deferred.promise();
	},



	// Run a lifetime method for a view
	// FLAG I want to be able to reverse the order of the functions (for unrender and kill)
	run: function (wrapper, method, order, content) {

		var deferred = $.Deferred();

		// Only proceed if view's not being processed
		if (p.views.status.get(wrapper)) {
			deferred.reject();
		} else {

			// Normalization & defaults
			if (!content) {
				content = {};
			}

			// Find the target containers
			var container = wrapper.children('.' + p.conf.views.classNames.view);
			var temp = p.views.detect(container);
			var type = temp.type;
			var view = temp.name;



			// Check out what methods are available
			// FLAG I need a more robust way to match the desired method names
			var i, key;
			var methods = [];
			var keys = {};
			
			// Global
			keys.global = bl.list.keys(app.life);
			for (i = 0; i < keys.global.length; i++) {
				key = keys.global[i];
				if (bl.has.value(method, key.split('_')) && bl.is.func(app.life[key])) {
					methods.push(app.life[key]);
				}
			}

			// Type-specific
			if (bl.is.hash(app.life.children[type])) {

				keys.type = bl.list.keys(app.life.children[type]);
				for (i = 0; i < keys.type.length; i++) {
					key = keys.type[i];
					if (bl.has.value(method, key.split('_')) && bl.is.func(app.life.children[type][key])) {
						methods.push(app.life.children[type][key]);
					}
				}

				// View-specific
				if (bl.is.hash(app.life.children[type].children[view])) {
					
					keys.view = bl.list.keys(app.life.children[type].children[view]);
					for (i = 0; i < keys.view.length; i++) {
						key = keys.view[i];
						if (bl.has.value(method, key.split('_')) && bl.is.func(app.life.children[type].children[view][key])) {
							methods.push(app.life.children[type].children[view][key]);
						}
					}

				}
			}

			if (order == 'reverse') {
				methods.reverse();
			}

			// Mark status of the view
			p.views.status.set(wrapper, method);



			// Fire events
			// FLAG I might not want this to happen here, might not want this at all, might want this to happen differently
			$(document).trigger(p.conf.views.classNames.status + '_' + method);
			$(document).trigger(p.conf.views.classNames.status + '_' + type + '_' + method);
			$(document).trigger(p.conf.views.classNames.status + '_' + type + '_' + view + '_' + method);



			// Run callback
			var _run = function (index) {
				var dfd = $.Deferred();

				// Run, if there are iterations left
				if (index < methods.length) {

					// Prepare parameters needed by view method
					var parameters = {
						app: p.app.get(),
						shared: bl.get.value(app.shared),
						view: {
							"type": type,
							"name": view,
							"container": container,
							"wrapper": wrapper,
							"content": content,
							"status": $.Deferred(),
							"selectors": bl.trim.empties(p.views.collect("dom", type, view))
						}
					};

					// Also add select jQuery object with selectors
					parameters.view.dom = p.dom.convert(parameters.view.selectors, container);

					methods[index](parameters.app, parameters.view, parameters.shared).then(function () {
						_run(index+1).then(function () {
							dfd.resolve();
						}, function () {
							dfd.reject();
						});
					});

				// No iterations left, we're done
				} else {
					dfd.resolve();
				}

				return dfd.promise();
			};

			// Execute!
			_run(0).then(function () {
				p.views.status.remove(wrapper, method);
				deferred.resolve();
			}, function () {
				deferred.reject();
			});

		}

		return deferred.promise();
	}

};
